#############################################################################
#    Ashtech remote receiver access tools
#    Copyright (C) 1999  Magellan Coropration/Ashtech Precision Products 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#
# FILE:        ash-lib.pl
#
# DESCRIPTION: This file contains library of  some useful subroutines
#              like Connect(), Query(), Command()
#
#  $Id$
#
#
#############################################################################

############################################################################
#
# Subroutine: Connect
#
# Description: This function establishes the connection to the 
#              receiver at the specified baud rate.
#
############################################################################

sub Connect {
    my ($DesiredBaudRate) = @_;
    my $hdr,$code,$port,$rate; # Used to parse the $PASHR,PRT response
    my $i,$k;
    my %BaudRates = (
		     5 => "9600",
		     7 => "38400",
		     9 => "115200",
		     6 => "19200",
		     8 => "57600"
		     );
    $MaxTries = 3;
    
    
# Set the 2 sec timeout on reading data from port
    system "stty time 20 min 0 -icanon";

# Walk through the baud rates to find current receiver rate
    $connected = 0;

    print STDERR "Try current baud rate\n";

  TRY: for($try=0;$try<$MaxTries;$try++)
  {
    CONNECT: for $i (5..10) {
	@Params = Query("PRT"); #Send $PASHQ,PRT and wait for $PASHR,RPT,x,y
	if ( $#Params >= 1 )    # The $PASHR,RPT,x,y was recived
	{
	    ($port,$rate) = @Params;   # Put x and y to $port,$rate vars
	    $connected = 1;
	    last TRY ;
	}	  
	
	print STDERR "Try ". $BaudRates{$i} . " baud\n";
	system  "stty " . $BaudRates{$i}; # Change the port baud rate
	sleep 1;                          # Wait for 1 sec
    }
  } 

    if ( $connected == 1 )
    {
	print STDERR "Found receiver at ", $BaudRates{$rate};
	print STDERR " bps, port ", $port ,"\n";

        # Set the detected  baud rate (it my differ form the current one) 
        # because of the misc delays

	system  "stty " . $BaudRates{$rate};
	sleep 1;
	
	# Set the desired baud rate

	Command("SPD," . $port . "," . $DesiredBaudRate);   
	system  "stty " . $BaudRates{$DesiredBaudRate};
	sleep 1;
	Query("PRT");
        print LOG "\nConnection: OK\n";
    }
    else
    {
        print LOG "\nConnection: NO\n";
    }

    return $connected;
}

############################################################################
#
# Subroutine: Query
#
# Description:
#
############################################################################
sub Query {
    my ($query) = @_;
    my $Line;
    
    # Build query string 
    $Line =  "\$PASHQ," . $query . "\n\r";
    print STDERR "< " , $Line; 

    # Send query command itself
    print $Line; 

    #Now wait for response $PASHR,xxx,
    ($QueryCode) = split /,/, $query;
    
    @Params = ();
  WaitForResponse: while (<STDIN>) {
      print STDERR ">", $_;
      # split the received line to header, command code, and parameters list
      ($hdr,$ReceivedCode,@TmpParams) = split /[,\*\n\r]/, $_; 
      if ( $ReceivedCode eq $QueryCode ) # Valid response found
      {
	  @Params = @TmpParams;
	  last WaitForResponse;
      }
  }
    
    return @Params; # return array of received response parameters
}

sub QuerySes {
    my ($query) = @_;
    my $Line;
    
    # Build query string 
    $Line =  "\$PASHQ," . $query . "\n\r";
    print STDERR "< " , $Line; 

    # Send query command itself
    print $Line; 

  WaitForResponse: while (<STDIN>) {
      print STDERR ">", $_;
      # split the received line to header, command code, and parameters list
      ($hdr,$flag,@TmpParams) = split ' ', $_; 
      if ($flag eq 'Y' || $flag eq 'N') 
      {
       $_ =~ tr/\r/ /; 
        print LOG "$_";
      }
      if ($flag eq 'REF' ) # Valid response found
      {
	  last WaitForResponse;
      }
  }
    
    return @Params; # return array of received response parameters
}


############################################################################
#
# Subroutine: Command
#
# Description: 
#
############################################################################
sub Command {
    my ($command,$DoCheckSum,$WaitAck) = @_;
    my $Line;

    # Build command string
    $Line = "\$PASHS," . $command;
    
    if( $DoCheckSum eq "yes" ) 
    {
	my($Dollar, @Body) = unpack("C*",$Line);
	my($Checksum);

	$Checksum = 0;
	for $i (0..$#Body) { $Checksum ^=  $Body[$i]; } 
	$Line = $Line . sprintf "*%02X",$Checksum; 
    }

    $Line = $Line  . "\n\r";

    print STDERR "< " , $Line; 
    # Send  command itself
    print $Line; 

    if ( $WaitAck eq "no" )
    {
	return "";
    }

  WaitForResponse: while (<STDIN>) {
      print STDERR ">", $_;
      ($hdr,$ReceivedCode) = split /[,\*]/, $_;
      if ( $ReceivedCode eq "ACK" ) {  return $ReceivedCode }
      if ( $ReceivedCode eq "NAK" ) {  return $ReceivedCode }
  }
    
    return "";
}

sub WaitFor {
    my ($Pattern,$TimeOut) = @_;


    $EndWaitTime = time() + 60 ; 
    
    $status = "";
    
  WaitForPatterm: while ( time() < $EndWaitTime )
  {
      while (<STDIN>) {
	  print STDERR "=>", $_;
	  return 1 if $_ eq $Pattern;
      }
  }

    return 0;
}

############################################################################
#
# Subroutine: ReadVar
#
# Description: Reads variable value from the configuration file. 
#              Configuration file looks like this:
#              ----------------------------------------------
#              # Comment line starts with '#'
#              VarName = VarValue   # Comment may follow 
#              ----------------------------------------------
#
# Parameters:  $FileName - Configuartion file name
#              $VarName  - Varaible name
#              $DefaultValue - assigned if file or variable not found
#
############################################################################

sub ReadVar {
    my ($FileName,$VarName,$DefaultValue) = @_;
    my  $ArgNum = $#_;
    my ( $Name, $Value) ;

    if ( ! open(CONF_FILE ,"$FileName") )  
    {
	if ($ArgNum == 1)  # No default value given 
	{
	    die "Can not open config file $FileName \n";
	}
	else              # Just use the default value
	{
	    return $DefaultValue;
	}
    }

  READ_LINE: while(<CONF_FILE>)
    {
	next READ_LINE if /^\#/ ;      # Skip comment lines

	# We don't was use the '=' as a separator, because it can be part
        # of the value (e.g., modem init string ). Simply replace 
        # the first occurence of it with '#' 

	$_ =~ s/=/\#/;                 # Replace the first '=' with '#'
	my @Pair  = split /\#/, $_ ;   # Split line to pair (Name,Value)

	next READ_LINE if $#Pair lt 1; # Not a valid line 

	foreach $Item (@Pair) {
	    $Item  =~ s/^\s+//;        # Remove leading  white spaces 
	    $Item  =~ s/\s+$//;        # Remove trailing white spaces 
	}

	($Name, $Value) = @Pair ; 

#	print "[$Name],[$Value]\n";
	last READ_LINE if $Name eq  $VarName;  # Found the variable name
    }

    close(CONF_FILE);

    return $Value        if $Name eq  $VarName;
    return $DefaultValue if $ArgNum == 2; 
    die  "$VarName was not found in file $FileName\n";
}

############################################################################
#
# Subroutine: GpsToNormalTime
#
# Description: Converts the GPS week and secons inside of that week to 
#             the normal calendar time. 
# Returns:
#  ($sec,$min,$hour,$mday,$mon,$year,$yday)
#
############################################################################
sub GpsToNormalTime
{
    my ($GPSWeek,$GPSSec) = @_;
    my ($sec,$min,$hour,$mday,$mon,$year,$yday);
    my @MonthDays=(31,28,31,30,31,30,31,31,30,31,30,31);
    my ($SecInDay,$SecInWeek,$SecIn4years,$FullTime,$DayNumber,$YearNumber,$month,$DayInYear );

    $SecInDay    = 24*3600;       # Seconds in day
    $SecInWeek   = 7*$SecInDay;   # Seconds in week
    $SecIn4years = 126230400;     # Seconds in 4-year interval 

    # Number of seconds since January, 6 1980
    $FullTime  = $GPSWeek * $SecInWeek + $GPSSec + 5*$SecInDay;
    $DayNumber  = int(int(($FullTime % $SecIn4years))/$SecInDay);
    
    $YearNumber = 0;
    if($DayNumber >= 366)
    {
	$DayNumber -= 366;
	$YearNumber += 1 + int($DayNumber/365);
	$DayNumber = int($DayNumber % 365);
    }
    $DayInYear = $DayNumber;

    if ($YearNumber eq 0) { $MonthDays[1] = 29;}  # February 

    FIND_MONTH: for($month=0; $month<12; $month++)
    {
        last FIND_MONTH if($DayNumber<$MonthDays[$month]) ;
        $DayNumber -= $MonthDays[$month];
    }

    $mon  = $month;
    $mday = $DayNumber+1;
    $year = int( 4*int($FullTime/$SecIn4years) + $YearNumber + 1980) % 100;
    $hour = int(int($FullTime % $SecInDay) / 3600);
    $min  = int(int($FullTime % 3600) / 60); 
    $sec  = int($FullTime % 60);
    $wday = int(int($FullTime / $SecInDay) % 7)+1;

    return ($sec,$min,$hour,$mday,$mon,$year,$wday,$DayInYear);
}


# This line has to be the very last line of the module, for some 
# obscured for me reason 
1; #return true , to please the require satement in main module




