#!/usr/bin/perl
#    Ashtech remote receiver access tools
#    Copyright (C) 1999  Magellan Coropration/Ashtech Precision Products 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#     FILE:        remote33.pl
#
#     DESCRIPTION: This PERL script downloads the data from  uZ,
#              conevrt it to RINEX format. The test 
#              This script has to be started like this:
#              ./remote33.pl <ConfiFile> <ReportFile> < /dev/ttyR0 > /dev/ttyR0
#              to have it standart inout and output redirected
#              to the  port connected to the modem or directly to receiver
#
#     $Id: remote33.pl,v 1.12 1999/11/09 20:57:40 serg Exp $
#
#
#############################################################################

require "ash-lib.pl";  # This library contains some useful subroutines
                         # like Connect(), Query(), Command()
require "mdm-lib.pl";  # This library contains some useful subroutines
                       # for all modem related scripts.

require "z12dld.pl";  # Z-12 download support 
                       

if ($#ARGV <1 ) {
  print STDERR "Correct usage is:\n";
  print STDERR "remote33.pl ConfigFile ReportFile\n";
  exit 1;
}

$ConfigFile = $ARGV[0];
$LogFile    = $ARGV[1];

print STDERR "Script params: $ConfigFile $LogFile\n";

# ------------------------------------------------------------------
# Read the configuration
# ------------------------------------------------------------------

$Title        = ReadVar($ConfigFile,"Title","No title specified");
$Connection   = ReadVar($ConfigFile,"Connection");
$ReceiverRate = ReadVar($ConfigFile,"ReceiverRate","7");
$Configure    = ReadVar($ConfigFile,"Configure","no");
$SessDuration = ReadVar($ConfigFile,"SessDuration",10);

if ( $Connection  eq  "modem" )  
{
    $BaudRate     = ReadVar($ConfigFile,"ConnectionRate",38400);
    $PhoneNumber  = ReadVar($ConfigFile,"PhoneNumber");
    $InitString   = ReadVar($ConfigFile,"PCModemInitString","ATS0=0S12=25&H1X4E0M3&D0&W");
}

$ScriptDir   = ReadVar($ConfigFile,"ScriptDir",".");
$ZmodemDir = ReadVar($ConfigFile,"ZmodemDir","$ScriptDir/zmodem/");
$ConvertDir = ReadVar($ConfigFile,"ZmodemDir","$ScriptDir/convert/");
$TeqcDir = ReadVar($ConfigFile,"ZmodemDir","$ScriptDir/teqc/");


$MNT_TIMEOUT=300; # Give maximum of 5 min for the data card to be mounted

$RCV_UNKN =  0;
$RCV_Z12 =  1;
$RCV_UZ =  2;
$ReceiverType = $RCV_UNKN;

print STDERR "Open log file\n";

open(LOG,">$LogFile") or AbortScript ("Can't open log file");

#
# Print the report header 
#

print LOG "Test Name: $Title \n";
$CurrTime = localtime();
print LOG $CurrTime, "\n";
$CurrDir=qx{pwd};
PrintStatus("Data Directory: $CurrDir\n");

# ------------------------------------------------------------------
# Configure the modem connected to the receiver
# ------------------------------------------------------------------

if ( $Configure eq "yes") 
{
    # We are going to setup the receiver modem before running the test
    # using one more serial port directly connected to the receiver

    PrintStatus ("Do the modem configuration\n");
    $rc = ConfigureReceiverModem();
    if ( $rc != 0 )
    {
	print LOG "Modem configuration has been changed !!!\n";
    }
    else
    {
	print LOG "Modem succesfully configured !!!\n";
    }
} 

# ------------------------------------------------------------------
# Connect to the receiver
# ------------------------------------------------------------------

if ( $Connection  eq  "direct" )
{
    PrintStatus ("Direct serial connection\n");
    $Connected =  Connect( $ReceiverRate );
}
elsif ( $Connection  eq  "modem" )  
{
    PrintStatus ("Dial phone $PhoneNumber to connect to receiver\n");
    HangUp ();
    $Connected =  not DialPhone ($BaudRate, $PhoneNumber, $InitString);
}
else  # It's perfectly legal. User just wanted initialize the modem
{
    PrintStatus ("No connection type was specfied\n");
    $Connected = 0;
}

if ( $Connected ) 
{
    PrintStatus ("Connected to receiver\n");
}
else
{
    AbortScript ("Connection to receiver failed\n");
}

# ------------------------------------------------------------------
# Obtain receiver identification
# ------------------------------------------------------------------

EstablishProtocol() or AbortScript( "Can not establish protocol");
GetReceiverId()     or AbortScript( "Can not identify the reciver");
PrepareReceiver()   or AbortScript( "Can not prepare reciver");

#----------------------------------------------------------------------------
# Session Program
#---------------------------------------------------------------------------

if( $ReceiverType eq $RCV_UZ )
{
    SetupSessions($SessDuration);
}
else
{
    SetupSessionsZ12($SessDuration);
}

#----------------------------------------------------------------------------
# Retrieve the list of files
#---------------------------------------------------------------------------

$TotalFiles = GetFilesList();
if ( $TotalFiles < 1 ) 
{
    AbortScript ("No files found in the receiver\n");
}  

#----------------------------------------------------------------------------
# Download files 
#----------------------------------------------------------------------------
 
$DownloadTries = 3;

DOWNLOAD_TRY: for($DldTry=0;$DldTry<$DownloadTries;$DldTry++)
{
    $DownLoadErr = DownloadFiles();
    last DOWNLOAD_TRY if not $DownLoadErr;

    if ( $Connection  eq  "modem" )  
    {
	HangUp ();
	$Connected =  not DialPhone ($BaudRate, $PhoneNumber, $InitString);
	last DOWNLOAD_TRY if not $Connected; # Something too wrong 
	EstablishProtocol() or AbortScript( "Can not establsih protocol");
	PrepareReceiver()   or AbortScript( "Can not prepare reciver");
    }
}

not $DownLoadErr or AbortScript( "Files Download failed after $DldTry attempts");

#-------------------------------------------------------------------------- 
# Delete downloaded files from the receiver
#-------------------------------------------------------------------------- 

if( $ReceiverType eq $RCV_Z12 )
{
    for($i=0; $i<$FilesFound;$i++)
    {
	Command("FIL,D,0","no","no"); # No checksum, no wait for $PASHR,ACK
	print STDERR "File was deleted\n" if WaitFor('C',3*60);  # Wai up to 3 min for 'C' character
    }
}

#----------------------------------------------------------------------------
# Process files 
#----------------------------------------------------------------------------

  ConvertAndCheckData();

#----------------------------------------------------------------------------
# Generate the report
#----------------------------------------------------------------------------

GenerateFilesReport($SessDuration);

$gmt = gmtime;
print LOG "\nCurr GMT time: $gmt\n";

if( $ReceiverType eq $RCV_UZ )
{
    ($GPSwk) = Query("WKN");
    print LOG "Curr GPS Week: $GPSwk\n";
}

print LOG "\nFiles were downloaded to the following dir:\n";
print LOG $CurrDir, "\n";


$URLbase = ReadVar($ConfigFile,"URLbase","http://mzserver.ashtech.com/");
@DirComponents =  split/\//,$CurrDir;
$BaseDir = $DirComponents[$#DirComponents]; 
$FullURL = $URLbase . $BaseDir;
print LOG "\These files can be obtained via HTTP at\n";
print LOG "$FullURL \n";

close(LOG);


if ( $Connection  eq  "modem" )  
{
    HangUp ();
}

1; 

############################################################################
#
# Subroutine: ConfigureReceiverModem
#
# Description: Runs the script to configure the receiver modem
#
# Parameters:  None
#
############################################################################

sub ConfigureReceiverModem
{
    $ScriptDir   = ReadVar($ConfigFile,"ScriptDir",".");
    $ScriptName  = $ScriptDir . "/setup_mdm.pl";
    $RcvrMdmPort = ReadVar($ConfigFile,"RcvrMdmPort","C"); 
    $ConfigPort  = ReadVar($ConfigFile,"ConfigPort"); 
    
    $Command = "perl -I $ScriptDir $ScriptName $RcvrMdmPort $ReceiverRate /dev/null < $ConfigPort > $ConfigPort";
    
    PrintStatus ("Use port $ConfigPort to configure receiver modem\n");
    PrintStatus ("Modem is connected to port $RcvrMdmPort of  receiver at rate $ReceiverRate\n") ;

    $rc = system $Command;
    return $rc;
}


############################################################################
#
# Subroutine:  EstablishProtocol
#
# Description: 
#
# Parameters:  None
#
############################################################################

sub EstablishProtocol {

    @Params = Query("PRT"); #Send $PASHQ,PRT and wait for $PASHR,RPT,x,y
    if ( $#Params < 1 )     # No response was received
    {
	# Receiver still may be in the ZMODMEM mode
	AbortZmodem();

	# Try again 
	@Params = Query("PRT"); #Send $PASHQ,PRT and wait for $PASHR,RPT,x,y
	return 0 if ( $#Params < 2 );
    }

    return 1;
}

############################################################################
#
# Subroutine:  PrepareReceiver
#
# Description: 
#
# Parameters:  None
#
############################################################################

sub PrepareReceiver {

    if( $ReceiverType eq $RCV_UZ )
    {
	Command("BEEP,OFF");
    
	$endtime = time()+$MNT_TIMEOUT;
      MNT_WAIT: while (time() < $endtime) {
	  ($Res) = Query("MNT");
	  last MNT_WAIT if ($Res eq '100%'); 
	  sleep 10;
      }
	
	sleep 1;
	
	return 1 if ($Res eq '100%'); 
	return 0;
    }

    return 1;
}


############################################################################
#
# Subroutine:  GetReceiverId
#
# Description: 
#
# Parameters:  None
#
############################################################################

sub GetReceiverId {
    
    EstablishProtocol() or return 0;

    @ver = Query("RID");

    if ($ver[0] eq "ZM" ) {$ReceiverType = $RCV_Z12;}
    if ($ver[0] eq "UZ" ) {$ReceiverType = $RCV_UZ;}
    
    print LOG "\nReceiver Firmware Ver: $ver[2]::$ver[4]\n";

    if( $ReceiverType eq $RCV_UZ )
    {
	print "\$PASHQ,SID\n\r";
	$timeout = time()+10;
    
      GetSer: while (<STDIN>) 
      {
	  ($serial) = $_;
	  if ($serial =~ /^SER/) {
	      last GetSer;
	  }
	  if (time() > $timeout) {
	      return 0;
	  }
      }
	
	$serial = substr($serial,6,15);

	print LOG "Receiver Serial No: $serial";
    }

    return 1;
}
############################################################################
#
# Subroutine:  GetFilesList
#
# Description: This subroutine obtains the list of the recorded files using 
#              the query $PASHQ,FIF,<n> where n is the index of the first 
#              file.
#              Each $PASHR,FIF can contain information for up to 10 files.
#              Format of the $PASHR,FIF is the following
#              $PASHR,FIF,<SpaceLeft>,<TotalFiles>,<FilesInThisMessage>,\
#              [<FileIdx>,<FileName>,<FileDate>,<FileSize>] - this group
#              is repeated <FilesInThisMessage> times
#
# Parameters:  None
#
############################################################################
sub GetFilesList
{

    if( $ReceiverType eq $RCV_Z12 )
    {
	return GetFilesListZ12();
    }

    $FilesFound = 0;
    $TotalFiles = 10;
    
  COLLECTDIR: while ( $FilesFound < $TotalFiles)
  {
      @Response = Query("FIF," . $FilesFound);
      
      if ( $#Response > 0 )
      {
	  ($BytesLeft,$TotalFiles,$FilesInThisQuery,@Files) = @Response;
	  for($i=0; $i<$FilesInThisQuery;$i++)
	  {
	      $FileName[$FilesFound] = $Files[$i*4+1];
	      $FileDate[$FilesFound] = $Files[$i*4+2];
	      $FileSize[$FilesFound] = $Files[$i*4+3];
	      $FileDeleted[$FilesFound] = 0;
	      $FilesFound ++;
	  }
      }
      else
      {
	  last COLLECTDIR;
      }
  }
    --$FilesFound; # Do not take in account last file, since it's open now
    return $FilesFound ;
}

############################################################################
#
# Subroutine:  DownloadFiles
#
# Description: This subroutines uses the utility rz to download receiver U
#              files via ZMODEM.
#              The recieiver initiates the transfer upon receiving command:
#              $PASHS,FIL,DOWNLOAD,<file name>,ZMODEM   
#
# Parameters:  None
#
############################################################################
sub DownloadFiles()
{
    if( $ReceiverType eq $RCV_Z12 )
    {
	return DownloadFilesFromZ12();
    }

    $DownloadError = 0;
    DOWNLOAD: for($i=0; $i<$FilesFound;$i++)
    {
	next DOWNLOAD if ( $FileDeleted[$i] );

	if ( Command("FIL,DOWNLOAD," . $FileName[$i] . ",ZMODEM") ne "NAK" )
	{
	    $DnStart = time();

	    # start rz utility, crash recovery set to ON
	    $DownloadError = system "$ConvertDir". "rz --resume --unrestrict";
	    last DOWNLOAD if $DownloadError;
	    
            # delete downloaded file from receiver
	    if ( Command ("FIL,D,0") eq "ACK" ) 
	    {
		$FileDeleted[$i] = 1;
	    }
	    $DnEnd = time();
	    $DownLoadTime[$i] = $DnEnd - $DnStart;
	}
    }
    return $DownloadError;
}
############################################################################
#
# Subroutine:  ConvertAndCheckData
#
# Description: This subroutines converts U-files to the RINEX format 
#              and get the measurement quality information 
#              out of these RINEX files.
# 
#              The conversion to the  RINEX is done in two steps:
#              a) u-convert utility converts U-files to B and E files
#              b) teqc      utility converts B and E files to RINEX
#
# Parameters:  None
#
############################################################################
sub ConvertAndCheckData()
{
    $DataDirName = "./";

    @ArchiveList = ();
    ($sec,$min,$hour,$mday,$mon,$year) = localtime();

    for($i=0; $i<$FilesFound ;$i++)
    {
	# Build necessary file names
	$UfileName        = $FileName[$i];
	$BaseFileName     =  substr($UfileName,1); # Strip leading 'U'
	$B_FileName       =  $DataDirName . "B"  .$BaseFileName;
	$RinexObsFileName =  $DataDirName . $BaseFileName . ".OBS";
	$RinexNavFileName =  $DataDirName . $BaseFileName . ".NAV";
	$ReportFileName   =  $DataDirName . $BaseFileName . "." . $year . "S";
	
	# Convert [U,R]-file to B,E,S files

	$ConvertCommand = $ConvertDir . "convert $UfileName  $DataDirName";

	print STDERR "Execute command: $ConvertCommand\n";
	system $ConvertCommand;

	@fileinfo = stat($B_FileName); 
	if ( int($fileinfo[7]) != 0 ) 
	{    
	
	    # Convert  B,E,S files to the RINEX files
	    $ConvertCommand = $TeqcDir . "teqc -ash d  +nav $RinexNavFileName +obs $RinexObsFileName $B_FileName";
	    print STDERR "Execute command: $ConvertCommand\n";
	    system $ConvertCommand;

	    # Remove zero size RINEX NAV file, otherwise the teqc will complain later
	    @fileinfo = stat($RinexNavFileName); 
	    unlink $RinexNavFileName if ( int($fileinfo[7]) == 0 ) ;
	
	    # Run Quality check for the RINEX file
	    $QaSummary[$i] = RunQualityCheck(  $RinexObsFileName ,$ReportFileName);
	    
	    # Addd files to the  list for the archiving
	    push ( @ArchiveList, $RinexObsFileName);
	    push ( @ArchiveList, $RinexNavFileName) unless ( int($fileinfo[7]) == 0 ) ;
	}
	else
	{
	    $QaSummary[$i] = "R-file Conversion error     ";
	}

	# Addd files to the  list for the archiving
	push ( @ArchiveList, $UfileName);
	# Delete temporary B,E,S files
	unlink $DataDirName . "B"  .$BaseFileName;
	unlink $DataDirName . "E"  .$BaseFileName;
	unlink $DataDirName . "S"  .$BaseFileName;
    }
}
#######################################################################################
#
# FUNCTION NAME: RunQualityCheck
# ARGUMENTS    : $Label,$Observables,$ReportFileName
# DESCRIPTION  : This function executes teqc utility to check the Quality of the 
#                data in the input RINEX file(s). It output the report like this:
#                U____B99.251 -> 01:26 01:30 1584/1582 (100%)
#                U____C99.251 -> 01:30 01:40 4186/4186 (100%)
#                U____D99.251 -> 01:40 01:50 4018/4186 (96%)
#                U____E99.251 -> 22:18 22:19 10/10 (100%)
#                U____F99.251 -> 22:56 22:58 28/30 (93%)
#                U____G99.251 -> 23:06 23:07 11/n/a (n/a%)
#                U____H99.251 -> 23:09 23:10 10/15 (67%)
#                U____I99.251 -> 23:11 23:13 26/30 (87%)
#                U____J99.251 -> 23:13 23:14 24/24 (100%)
#                Each line consists from the following fields:
#                1. Label
#                2. Time of the first epoch
#                3. Time of the last epoch
#                4. Number of observables in the file
#                5. Potential  number of observables in the file 
#                          calculated on base of the ephemeris 
#                          information
#                Ratio of the  last two fields.
#
#######################################################################################
sub RunQualityCheck
{
    my ($Observables,$ReportFileName) = @_ ;
    my ($Keyword, $StartYear, $StartMonth, $StartDay,  $StartTime,
        $EndYear, $EndMonth,  $EndDay,     $EndTime,
	$Length,  $Interval,  $ExpObs,     $HaveObs,    $PercObs) ;
    my ($Command) ;

# Execute teqc to generate the report file
    $Command = $TeqcDir . "teqc +qc -plot $Observables >/dev/null";
    print STDERR "Execute command: $Command\n";
    system $Command;

# Process Report File

    open(REPORT_FILE,"<$ReportFileName") or AbortScript ("Can not open $ReportFileName");

  PROCESS_REPORT: while( <REPORT_FILE> )
    {
	@LineItems = split( ' ',$_);
	last PROCESS_REPORT if ( $LineItems[0] eq "SUM" ) ; # Find the summary line
    }
    close (REPORT_FILE);

    $Keyword = "";

     # Split the summary line to the items
    ($Keyword, $StartYear, $StartMonth, $StartDay,  $StartTime,
               $EndYear,   $EndMonth,   $EndDay,    $EndTime,
     $Length,  $Interval,  $ExpObs,     $HaveObs,   $PercObs) = @LineItems;
    
     if ( $Keyword eq "SUM")
     {
	 $Summ =  "$StartTime $EndTime $HaveObs/$ExpObs ($PercObs%)";
     }
     else
     {
	 $Summ = "No report Was generated";
     }
    return $Summ;
}

############################################################################
#
# Subroutine:  GenerateFilesReport
#
# Description: This subroutines generates the report on the files quality.
#
#
# Parameters:  None
#
############################################################################

sub GenerateFilesReport()
{
    my ($SessInfo) = @_;

    $last_hour=0;
    $last_minu=0;
    print LOG "Name         Week Day Time  Rcv/Dnld      First Last   Obs Ratio       DnSpd   Notes:\n";

    for($i=0; $i<$FilesFound;$i++)
    {
	$Notes = ' ';
	@fileinfo = stat($FileName[$i]);
	$dn_sz = int($fileinfo[7]/1024+0.5);

	$DnSpeed = 0;
	if ($DownLoadTime[i] != 0 ) 
	{
	    $DnSpeed = int($fileinfo[7]*10/$DownLoadTime[i]);
	}


	%wkdays = qw/1 Sun 2 Mon 3 Tue 4 Wed 5 Thu 6 Fri 7 Sat/;

	if( $ReceiverType eq $RCV_UZ )
	{
	    $week = substr($FileDate[$i],0,5);
	    $day = $wkdays{chop($week)};
	    $hour = substr($FileDate[$i],5,8);
	    $hour = substr($hour,0,2);
	    $minu = substr($FileDate[$i],7,8);
	}
	
	if( $ReceiverType eq $RCV_Z12 )
	{
	    $week = $FileStartWn[$i];
	    ($sec,$minu,$hour,$mday,$mon,$year,$day,$yday) = GpsToNormalTime($FileStartWn[$i],$FileStartTime[$i]);
	    $day = $wkdays{chop($day)};
	    $hour = sprintf("%02d",$hour);
	    $minu = sprintf("%02d",$minu);
	    $FileSize[$i] = sprintf("%06d",int($FileSize[$i]/1024+0.5));
	}

	$sessdiff = ($minu-$last_minu) + 60*($hour-$last_hour);
	$last_hour=$hour;
	$last_minu=$minu;
	
	$Notes = "";

	if( $ReceiverType eq $RCV_UZ )
	{
	    if ($i>0 && ($sessdiff != $SessInfo)) {
		$Notes = '*** Time Err ';
	    }
	}

	if ($dn_sz !=  $FileSize[$i]) {
	    $Notes = $Notes .  ' *** Size Err';
	}

	$dn_sz = sprintf("%06d", $dn_sz);
	print LOG "$FileName[$i] $week $day $hour:$minu $FileSize[$i]/$dn_sz $QaSummary[$i]  $DnSpeed   $Notes\n";
    }

    # Compress data files 
    foreach $FileName (@ArchiveList) 
    {
	system "gzip $FileName";
    }
}

############################################################################
#
# Subroutine:  SetUpSessions
#
# Description: 
#
# Parameters:  Session duration in minutes
#
############################################################################
sub SetupSessions 
{
    my ($SessInfo) = @_;

    $iha = int($SessInfo/60);
    $ima = $SessInfo - $iha*60;

    $ses_nm='A';
    $ses_flg='Y';
    $init=0;
    $end=1;
    if ($SessInfo == 60) {
	$ses_int=10;
    }
    else {
	$ses_int=1;
    }
    $ses_mask=0;
    $ses_min=0;
    $ses_type=0;
    $sw=0;
    $ih=0;
    $eh=0;
    $reprg=0;


    print LOG "\nSessions:\n";
    print LOG "    Start    End      RCI   Mask MIN TYPE\n";
    for $j(0..23) {
	@Rspn = Query("SSN," . chr($j+65));
	if ($#Rspn > 0) {
	    for $i(5..12) {
		if ($i==12) {
		    $Rspn[$i] =~ tr/\r/ /;
		    print LOG "$Rspn[$i]";
		}
		else {
		    print LOG "$Rspn[$i] ";
		}
	    }
	}
	($sh,$sm,$ss) = split /:/, $Rspn[7];
	($fh,$fm,$fs) = split /:/, $Rspn[8];
	$start_time = 3600*$sh+60*$sm+$ss;
	$end_time = 3600*$fh+60*$fm+$fs;
	if ($j==0) {
	    $first_starthr = $sh; 
	}
	if ($end_time == 0 && $end_time < $start_time) {
	    $end_time = 3600*24+60*$fm+$fs;
	}
	$duration = $end_time - $start_time +1;
	if ($reprg==0){
	    if (($duration != 60*$SessInfo) || (int($Rspn[9]) != $ses_int) || (int($Rspn[10]) != $ses_mask) || ($Rspn[11] != $ses_min)){
		$reprg=1;
	    }
	}
    }
    
    @GMtime = gmtime(time);
    $ih = sprintf("%02d",$GMtime[2]+1);
    $im = '00';
    $eh = $ih + $iha;
    $em = $im + $ima;
    print STDERR "ih = $ih, eh = $eh, sh = $first_starthr \n";
    if ($SessInfo != 60){
	if (($ih-$first_starthr)>3) {
	    $reprg=2;
	}
    }
 
#----------------------------------------------------------------------------
# If session pattern not match, then reprogram sessions
#---------------------------------------------------------------------------
    if ($reprg > 0) {
	print STDERR "reprg = $reprg \n";  
	Command("SES,DEL");
	Command("SES,PAR,Y,257,0");
	for $i(0..23){
	    $sw = 1;
	    if ($ih >= 24) {
		$ih -= 24;
	    } 
	    if ($eh >= 24) {
		$eh -= 24;
	    } 
	    if ($im >= 60) {
		$im -= 60;
		$ih++; 
	    } 
	    if ($em >= 60) {
		$em -= 60;
		$eh++; 
     } 
	    $ih = sprintf("%02d",$ih);
	    $im = sprintf("%02d",$im);
	    $eh = sprintf("%02d",$eh);
	    $em = sprintf("%02d",$em);
	    Command("SES,SET," . chr($i+65) . "," . $ses_flg . "," . $ih . $im . "0" . $sw . "," . $eh . $em . "00," . $ses_int . "," . $ses_mask . "," . $ses_min . "," . $ses_type);
	    $ih = $ih + $iha;
	    $im = $im + $ima;
	    $eh += $iha;
	    $em += $ima;
	}
	print LOG "\nReprogramed Sessions:\n";
	print LOG "    Start    End      RCI   Mask MIN TYPE\n";
	@Response00 = QuerySes("SES");
    }  

    Command("SAV,Y");
}

############################################################################
#
# Subroutine:  PrintStatus
#
# Description: 
#
# Parameters:  
#
############################################################################
sub PrintStatus
{
    print STDERR @_;
}
############################################################################
#
# Subroutine:  AbortZmodem
#
# Description: Abort the ZMODEM transfer by sending the series of ZCAN bytes
#
# Parameters:  
#
############################################################################
sub AbortZmodem
{
    PrintStatus("Send ZMODEM abort string\n");
    $AbortString = "\x18\x18\x18\x18\x18\x18\x18\x18\x18\x18\x18\x18\n";
    print $AbortString;
    print STDERR $AbortString;
    sleep ( 20 );
}

############################################################################
#
# Subroutine:  AbortScript
#
# Description: Aborts the script
#
# Parameters:  
#
############################################################################
sub AbortScript
{
    print LOG @_ , "\n";
    print STDERR @_, "\n";

    print LOG "Test aborted\n";

    close(LOG);
    die "Test aborted";
}
